@@ -598,17 +598,16 @@ WHERE
 
         timezone = evt[KG__security_event__creation_timestamp].tzinfo
         add_seconds = timedelta(seconds=evt[KG__application__unlock_code_validity_period])
-        print(timezone)
-        print(evt[KG__security_event__creation_timestamp])
-        print(add_seconds)
-        print(evt[KG__security_event__creation_timestamp] + add_seconds)
-        print(datetime.now(timezone))
         if evt[KG__security_event__creation_timestamp] + add_seconds < datetime.now(timezone):
             raise HttpStatusException(ERR__unlock_code_expired)
 
         if not evt[KEY__key_fits]:
             raise HttpStatusException(ERR__incorrect_lock_code)
 
+        if evt[KG__security_event__unlock_timestamp] is None:
+            security_event__update(self.jaaql_lookup_connection, self.get_db_crypt_key(), evt[KG__security_event__application],
+                                   evt[KG__security_event__event_lock], unlock_timestamp=datetime.now())
+
         if returning:
             return evt
         else:
@@ -616,10 +615,6 @@ WHERE
                                               evt[KG__security_event__email_template])
             return template[KG__email_template__type]
 
-    def mark_security_event_unlocked(self, sec_evt: dict):
-        security_event__update(self.jaaql_lookup_connection, sec_evt[KG__security_event__application], sec_evt[KG__security_event__event_lock],
-                               unlock_timestamp=datetime.now())
-
     def finish_security_event(self, inputs: dict):
         sec_evt = self.check_security_event_key_and_security_event_is_unlocked({
             KG__security_event__event_lock: inputs[KG__security_event__event_lock],
@@ -633,15 +628,11 @@ WHERE
 
         parameters = None
 
-        if template[KG__email_template__data_validation_table]:
-            # app = application__select(self.jaaql_lookup_connection, sec_evt[KG__security_event__application])  # TODO try select parameters
-            parameters = {}
-            # TODO possibly load parameters
-
         if template[KG__email_template__type] in [EMAIL_TYPE__signup, EMAIL_TYPE__reset_password, EMAIL_TYPE__unregistered_password_reset]:
             self.add_account_password(sec_evt[KG__security_event__account], inputs[KEY__password])
 
-        self.mark_security_event_unlocked(sec_evt)
+        security_event__update(self.jaaql_lookup_connection, self.get_db_crypt_key(), sec_evt[KG__security_event__application],
+                               sec_evt[KG__security_event__event_lock], finish_timestamp=datetime.now())
 
         # TODO maybe someday we'll add this back in
         # if template[KG__email_template__type] == EMAIL_TYPE__signup:
@@ -690,22 +681,25 @@ WHERE
             KEY__schema: fetched_template[KG__email_template__validation_schema]
         }
 
-        data_relation = fetched_template[KG__email_template__data_validation_view]
+        data_relation = fetched_template[KG__email_template__permissions_and_data_view]
         if re.match(REGEX__dmbs_object_name, data_relation) is None:
-            raise HttpStatusException("Unsafe data relation specified for email template")
+            raise HttpStatusException("Unsafe " + KG__email_template__permissions_and_data_view + " specified for email template")
         safe_parameters = []
         for key, val in inputs[KEY__parameters].items():
             if re.match(REGEX__dmbs_object_name, key) is None:
                 raise HttpStatusException("Unsafe parameter specified for email template")
             safe_parameters.append(f'{key} = :{key}')  # Ignore pycharm PEP issue
         where_clause = " AND ".join(safe_parameters)
-        submit_data[KEY_query] = f'SELECT * FROM {data_relation} WHERE {where_clause}'  # Ignore pycharm PEP issue
+        if len(where_clause) != 0:
+            where_clause = " WHERE " + where_clause
+        submit_data[KEY_query] = f'SELECT * FROM {data_relation}{where_clause}'  # Ignore pycharm PEP issue
         # We now get the data that can be shown in the email
         email_replacement_data = submit(self.vault, self.config, self.get_db_crypt_key(), self.jaaql_lookup_connection,
                                         submit_data, account_id, None, self.cached_canned_query_service,
                                         as_objects=True, singleton=True)
 
-        document_templates = fetch_document_templates_for_email_template(self.jaaql_lookup_connection, inputs[KEY__application], inputs[KEY__template])
+        document_templates = fetch_document_templates_for_email_template(self.jaaql_lookup_connection, inputs[KEY__application],
+                                                                         inputs[KEY__template])
         attachments = [
             EmailAttachment(
                 template[KG__document_template__name], template[KG__document_template__application], inputs[KEY__parameters],
@@ -718,16 +712,13 @@ WHERE
         if self.is_container:
             attachment_base_url = app[KG__application__base_url]
 
-        self.email_manager.construct_and_send_email(app[KG__application__artifacts_source],
+        self.email_manager.construct_and_send_email(app[KG__application__templates_source],
                                                     fetched_template[KG__email_template__dispatcher], fetched_template,
                                                     username, email_replacement_data,
                                                     attachments=attachments, attachment_access_token=auth_token,
                                                     attachment_base_url=attachment_base_url)
 
-    def reset_password(self, inputs: dict, is_the_anonymous_user: bool):
-        if is_the_anonymous_user:
-            raise HttpStatusException("Cannot change this user's password")
-
+    def reset_password(self, inputs: dict):
         app = application__select(self.jaaql_lookup_connection, inputs[KG__security_event__application])
         if inputs[KEY__reset_password_template] is None:
             inputs[KEY__reset_password_template] = app[KG__application__default_r_et]
@@ -751,53 +742,84 @@ WHERE
             raise HttpStatusException(ERR__template_not_unregistered)
 
         account_existed = False
-        try:
-            account_id = self.create_account_with_potential_password(self.jaaql_lookup_connection, inputs[KEY__username])
-        except HttpStatusException as hs:
-            if not hs.message.startswith(SQL__err_duplicate):
-                raise hs  # Unrelated exception, raise it
 
+        # TODO allow create account here under example
+        # You don't have an account with us, would you like one?
+        # Think about how that might impact fake account
+
+        account_id = None
+        fake_account_username = None
+
+        try:
             account = fetch_account_from_username(self.jaaql_lookup_connection, self.get_db_crypt_key(), self.get_vault_repeatable_salt(),
                                                   inputs[KEY__username])
             account_id = account[KG__account__id]
             if account[KG__account__most_recent_password] is not None:
                 account_existed = True
 
-        count = execute_supplied_statement_singleton(self.jaaql_lookup_connection,
-                                                     QUERY__count_security_events_of_type_in_24hr_window,
-                                                     {
-                                                         KEY__type_one: EMAIL_TYPE__reset_password,
-                                                         KEY__type_two: EMAIL_TYPE__unregistered_password_reset,
-                                                         KG__security_event__account: account_id
-                                                     }, as_objects=True)[ATTR__count]
+            jaaql_singleton = jaaql__select(self.jaaql_lookup_connection)
+
+            if account == jaaql_singleton[KG__jaaql__the_anonymous_user]:
+                raise HttpStatusException("Cannot change this user's password as this user is the anonymous user")
+        except HttpSingletonStatusException:
+            fake_account_username = inputs[KEY__username]
+
+        count = count_for_security_event(self.jaaql_lookup_connection, self.get_db_crypt_key(), self.get_vault_repeatable_salt(),
+                                         EMAIL_TYPE__reset_password, EMAIL_TYPE__unregistered_password_reset,
+                                         account_id, fake_account_username)
 
         if count >= RESEND__reset_max:
             raise HttpStatusException(ERR__too_many_reset_requests, HTTPStatus.TOO_MANY_REQUESTS)
 
-        template = None
-        if account_existed and inputs[KEY__reset_password_template] is not None:
-            template = inputs[KEY__reset_password_template]
-        elif not account_existed and inputs[KEY__unregistered_user_reset_password_template] is not None:
-            template = inputs[KEY__unregistered_user_reset_password_template]
-
+        template = reset_template if account_existed else unregistered_template
         unlock_code = self.gen_security_event_unlock_code(CODE__letters, CODE__reset_length)
-        reg_env_ins = security_event__insert(self.jaaql_lookup_connection, inputs[KG__security_event__application], template, account_id,
-                                             unlock_code)
+        reg_env_ins = security_event__insert(self.jaaql_lookup_connection, self.get_db_crypt_key(),
+                                             inputs[KG__security_event__application], template[KG__email_template__name], unlock_code,
+                                             account_id, fake_account_username, encryption_salts={
+                KG__security_event__fake_account: get_repeatable_salt(self.get_vault_repeatable_salt(), fake_account_username)})
 
-        self.email_manager.send_email(self.vault, self.config, self.get_db_crypt_key(), self.jaaql_lookup_connection,
-                                      inputs[KG__security_event__application], template,
-                                      app[KG__application__artifacts_source],
-                                      app[KG__application__base_url], account_id, inputs[KEY__parameters],
-                                      parameter_id=reg_env_ins[KG__security_event__event_lock], none_sanitized_parameters={
-                EMAIL_PARAM__unlock_key: reg_env_ins[KG__security_event__unlock_key],
-                EMAIL_PARAM__unlock_code: unlock_code
-            })
+        sign_up_perms_data_view = template[KG__email_template__permissions_and_data_view]
+        if re.match(REGEX__dmbs_object_name, sign_up_perms_data_view) is None:
+            raise HttpStatusException("Unsafe data relation specified for sign up")
+        reset_password_data = None
+        try:
+            if account_existed:
+                submit_data = {
+                    KEY__application: inputs[KG__security_event__application],
+                    KEY_query: f"SELECT * FROM {sign_up_perms_data_view}",
+                    KEY__schema: template[KG__email_template__validation_schema]
+                }
+                account_db_interface = get_required_db(self.vault, self.config, self.jaaql_lookup_connection, submit_data, account_id)
+                reset_password_data = submit(self.vault, self.config, self.get_db_crypt_key(), self.jaaql_lookup_connection, submit_data, account_id,
+                                             None, self.cached_canned_query_service, interface=account_db_interface, as_objects=False, singleton=True)
+                reset_password_data = objectify(reset_password_data, singleton=True)
+            else:
+                submit_data = {
+                    KEY__application: inputs[KG__security_event__application],
+                    KEY__schema: template[KG__email_template__validation_schema],
+                    KEY_query: f'SELECT * FROM {sign_up_perms_data_view}'  # Ignore pycharm PEP issue
+                }
+
+                reset_password_data = submit(self.vault, self.config, self.get_db_crypt_key(),
+                                             self.jaaql_lookup_connection, submit_data, self.jaaql_lookup_connection.role,
+                                             None, self.cached_canned_query_service, as_objects=True, singleton=True)
+
+        except HttpSingletonStatusException:
+            raise HttpSingletonStatusException("Multiple rows returned when selecting from reset password data view")
+
+        reset_password_data[EMAIL_PARAM__app_url] = app[KG__application__base_url]
+        reset_password_data[EMAIL_PARAM__email_address] = inputs[KEY__username]
+        reset_password_data[EMAIL_PARAM__unlock_key] = reg_env_ins[KG__security_event__unlock_key]
+        reset_password_data[EMAIL_PARAM__unlock_code] = unlock_code
+
+        self.email_manager.construct_and_send_email(app[KG__application__templates_source], template[KG__email_template__dispatcher], template,
+                                                    inputs[KEY__username], reset_password_data)
 
         return {
             KG__security_event__event_lock: reg_env_ins[KG__security_event__event_lock]
         }
 
-    def sign_up(self, inputs: dict, account_id: str, is_the_anonymous_user: bool):
+    def sign_up(self, inputs: dict, account_id: str):
         app = application__select(self.jaaql_lookup_connection, inputs[KG__security_event__application])
 
         if inputs[KEY__sign_up_template] is None:
@@ -823,10 +845,6 @@ WHERE
         if already_signed_up_template[KG__email_template__type] != EMAIL_TYPE__already_signed_up:
             raise HttpStatusException(ERR__template_not_already)
 
-        if is_the_anonymous_user and (not already_signed_up_template[KG__email_template__can_be_sent_anonymously] or
-                                      not sign_up_template[KG__email_template__can_be_sent_anonymously]):
-            raise HttpStatusException("Cannot sign up publicly using this route")
-
         conn = None
         account_db_interface = None
 
@@ -834,39 +852,59 @@ WHERE
             submit_data = {
                 KEY__application: inputs[KG__security_event__application],
                 KEY__parameters: inputs[KEY__parameters],
-                KEY_query: inputs[KEY_query],
+                KEY_query: inputs.get(KEY_query),
                 KEY__schema: sign_up_template[KG__email_template__validation_schema]
             }
 
             account_db_interface = get_required_db(self.vault, self.config, self.jaaql_lookup_connection, submit_data, account_id)
             conn = account_db_interface.get_conn()
 
-            ret = submit(self.vault, self.config, self.get_db_crypt_key(), self.jaaql_lookup_connection, submit_data, account_id, None,
-                         self.cached_canned_query_service, as_objects=False, singleton=True, keep_alive_conn=True, conn=conn, interface=account_db_interface)
-            ret = objectify(ret[list(ret.keys())[-1]], singleton=True)
+            pre_ret = None
+
+            if inputs.get(KEY_query):
+                # For public sign up perform this query as the public user
+                ret = submit(self.vault, self.config, self.get_db_crypt_key(), self.jaaql_lookup_connection, submit_data, account_id,
+                             None,
+                             self.cached_canned_query_service, as_objects=False, singleton=True, keep_alive_conn=True, conn=conn,
+                             interface=account_db_interface)
+                pre_ret = ret
+                ret = objectify(ret[list(ret.keys())[-1]], singleton=True)
+            else:
+                # For public sign up be careful about this
+                ret = inputs[KEY__parameters]
+
             # This is the permissions check. It is an update that returns something that is ignored
             # An exception will be triggered here if the user does not have permissions
             where_clause = " AND ".join(['"' + key + '" = :' + key for key in ret.keys() if re.match(REGEX__dmbs_object_name, key) is not None])
-            if len(where_clause) != 0:
-                where_clause = " AND " + where_clause
-            sign_up_table = sign_up_template[KG__email_template__data_validation_table]
-            upt_query = f'UPDATE "{sign_up_table}" SET "{dbms_user_column_name}" = :{dbms_user_column_name} WHERE {dbms_user_column_name} is null{where_clause}'  # Ignore pycharm pep issue
-            submit_data[KEY_query] = upt_query
+            where_clause = " WHERE " + where_clause
+            sign_up_perms_data_view = sign_up_template[KG__email_template__permissions_and_data_view]
+            if re.match(REGEX__dmbs_object_name, sign_up_perms_data_view) is None:
+                raise HttpStatusException("Unsafe data relation specified for sign up")
+            sign_up_perms_data_query = f'SELECT * FROM "{sign_up_perms_data_view}"{where_clause}'  # Ignore pycharm pep issue
+            submit_data[KEY_query] = sign_up_perms_data_query
             submit_data[KEY_parameters] = ret
-            submit_data[KEY_parameters][dbms_user_column_name] = None
-            submit(self.vault, self.config, self.get_db_crypt_key(), self.jaaql_lookup_connection, submit_data, account_id,
-                   None, self.cached_canned_query_service, keep_alive_conn=True, conn=conn, interface=account_db_interface)
 
-            data_relation = sign_up_template[KG__email_template__data_validation_view]
-            if data_relation is None:
-                data_relation = sign_up_template[KG__email_template__data_validation_table]
+            try:
+                sign_up_data = submit(self.vault, self.config, self.get_db_crypt_key(), self.jaaql_lookup_connection, submit_data, account_id,
+                                      None, self.cached_canned_query_service, keep_alive_conn=True, conn=conn,
+                                      interface=account_db_interface, as_objects=False, singleton=True)
+                sign_up_data = objectify(sign_up_data, singleton=True)
+                if dbms_user_column_name in sign_up_data:
+                    raise HttpStatusException("Security alert! Dbms user column name " + dbms_user_column_name + " cannot be present in view " +
+                                              sign_up_perms_data_view)
+            except HttpSingletonStatusException:
+                raise HttpSingletonStatusException("No or multiple rows returned from " + sign_up_perms_data_view + " with " + where_clause)
+
+            base_relation = sign_up_template[KG__email_template__base_relation]
+            if re.match(REGEX__dmbs_object_name, base_relation) is None:
+                raise HttpStatusException("Unsafe data relation specified for sign up")
 
             if inputs.get(KEY__username) is None:
                 get_user_data = {
                     KEY__application: inputs[KG__security_event__application],
                     KEY__schema: sign_up_template[KG__email_template__validation_schema],
-                    KEY_parameters: inputs[KEY__parameters],
-                    KEY_query: f'SELECT {dbms_user_column_name} FROM {data_relation} WHERE {where_clause[5:]}'  # Ignore pycharm PEP issue
+                    KEY_parameters: ret,
+                    KEY_query: f'SELECT {dbms_user_column_name} FROM {base_relation}{where_clause}'  # Ignore pycharm PEP issue
                 }
 
                 try:
@@ -895,20 +933,14 @@ WHERE
                 if account[KG__account__most_recent_password] is not None:
                     account_existed = True
 
-            # We set the dbms user in the application. This can potentially cause an index clash due to a unique index on the dbms_user column
-            # This is desirable in many situations but will allow username enumeration
-            submit_data[KEY__parameters][dbms_user_column_name] = new_account_id
-            submit(self.vault, self.config, self.get_db_crypt_key(), self.jaaql_lookup_connection, submit_data, account_id,
-                   None, self.cached_canned_query_service, keep_alive_conn=True, conn=conn, interface=account_db_interface)
+                jaaql_singleton = jaaql__select(self.jaaql_lookup_connection)
+
+                if account == jaaql_singleton[KG__jaaql__the_anonymous_user]:
+                    raise HttpStatusException("Cannot re-invite the anonymous user")
 
             # We abort if there have been too many requests. Everything is rolled back
-            count = execute_supplied_statement_singleton(self.jaaql_lookup_connection,
-                                                         QUERY__count_security_events_of_type_in_24hr_window,
-                                                         {
-                                                             KEY__type_one: EMAIL_TYPE__signup,
-                                                             KEY__type_two: EMAIL_TYPE__already_signed_up,
-                                                             KG__security_event__account: new_account_id
-                                                         }, as_objects=True)[ATTR__count]
+            count = count_for_security_event(self.jaaql_lookup_connection, self.get_db_crypt_key(), self.get_vault_repeatable_salt(),
+                                             EMAIL_TYPE__signup, EMAIL_TYPE__already_signed_up, new_account_id)
             if count >= RESEND__invite_max:
                 raise HttpStatusException(ERR__too_many_signup_attempts, HTTPStatus.TOO_MANY_REQUESTS)
 
@@ -916,38 +948,34 @@ WHERE
             # If this fails we are unfortunately up shit creek without a paddle but this can't fail if the application is designed correctly
             account_db_interface.put_conn_handle_error(conn, None)  # No errors. This will commit
 
-            # Choose the relation. We are deliberately going from the sign-up template ONLY to reduce the scope of what can go wrong security wise
-            # This is because a user can mix and match sign up and already signed up, potentially leaking data
-            # We will change the model in the future when time allows that we have a single sign up and reset template with alternatives attached
-            if re.match(REGEX__dmbs_object_name, data_relation) is None:
-                raise HttpStatusException("Unsafe data relation specified for sign up")
-            submit_data[KEY_query] = f'SELECT * FROM {data_relation} WHERE "{dbms_user_column_name}" = :{dbms_user_column_name}{where_clause}'  # Ignore pycharm PEP issue
-            # We now get the data that can be shown in the email
-            try:
-                email_replacement_data = submit(self.vault, self.config, self.get_db_crypt_key(), self.jaaql_lookup_connection, submit_data,
-                                                self.jaaql_lookup_connection.role, None, self.cached_canned_query_service, as_objects=True,
-                                                singleton=True)
-            except HttpSingletonStatusException:
-                del submit_data[KEY__parameters][dbms_user_column_name]
-                raise HttpSingletonStatusException(f'Unable to locate email data. Please check that the data returned from your input query (do _not_ include {
-                    dbms_user_column_name}) matches an entry in {data_relation}: ' + json.dumps(submit_data, indent=4))
+            # We set the dbms user in the application. This can potentially cause an index clash due to a unique index on the dbms_user column
+            # This is desirable in many situations but will allow username enumeration
+            submit_data[KEY_query] = f"UPDATE {base_relation} SET {dbms_user_column_name} = :{dbms_user_column_name}{where_clause}"
+            submit_data[KEY__parameters][dbms_user_column_name] = new_account_id
+            submit(self.vault, self.config, self.get_db_crypt_key(), self.jaaql_lookup_connection, submit_data, self.jaaql_lookup_connection.role,
+                   None, self.cached_canned_query_service)
 
             template = already_signed_up_template if account_existed else sign_up_template
             unlock_code = self.gen_security_event_unlock_code(CODE__letters, CODE__invite_length)
-            reg_event = security_event__insert(self.jaaql_lookup_connection, inputs[KG__security_event__application], template[KG__email_template__name],
-                                               new_account_id, unlock_code)
 
-            email_replacement_data[EMAIL_PARAM__app_url] = app[KG__application__base_url]
-            email_replacement_data[EMAIL_PARAM__email_address] = inputs[KEY__username]
-            email_replacement_data[EMAIL_PARAM__unlock_key] = reg_event[KG__security_event__unlock_key]
-            email_replacement_data[EMAIL_PARAM__unlock_code] = unlock_code
+            reg_event = security_event__insert(self.jaaql_lookup_connection, self.get_db_crypt_key(),
+                                               inputs[KG__security_event__application], template[KG__email_template__name], unlock_code,
+                                               new_account_id)
 
-            self.email_manager.construct_and_send_email(app[KG__application__artifacts_source], template[KG__email_template__dispatcher], template,
-                                                        inputs[KEY__username], email_replacement_data)
+            sign_up_data[EMAIL_PARAM__app_url] = app[KG__application__base_url]
+            sign_up_data[EMAIL_PARAM__email_address] = inputs[KEY__username]
+            sign_up_data[EMAIL_PARAM__unlock_key] = reg_event[KG__security_event__unlock_key]
+            sign_up_data[EMAIL_PARAM__unlock_code] = unlock_code
 
-            return {
-                KG__security_event__event_lock: reg_event[KG__security_event__event_lock]
-            }
+            self.email_manager.construct_and_send_email(app[KG__application__templates_source], template[KG__email_template__dispatcher], template,
+                                                        inputs[KEY__username], sign_up_data)
+
+            if pre_ret is not None:
+                return pre_ret
+            else:
+                return {
+                    KG__security_event__event_lock: reg_event[KG__security_event__event_lock]
+                }
         except Exception as err:
             if conn is not None:
                 account_db_interface.put_conn_handle_error(conn, err)
